part of forestvpn_api.api;

class AppleStoreServerNotification {
  /* An identifier that App Store Connect generates and the App Store uses to uniquely identify the auto-renewable subscription that the user's subscription renews. Treat this value as a 64-bit integer. */
  String autoRenewAdamId = null;
  
/* The product identifier of the auto-renewable subscription that the user's subscription renews. */
  String autoRenewProductId = null;
  
/* The current renewal status for an auto-renewable subscription product. Note that these values are different from those of the auto_renew_status in the receipt. */
  String autoRenewStatus = null;
  //enum autoRenewStatusEnum {  true,  false,  };
/* The time at which the renewal status for an auto-renewable subscription was turned on or off, in a date-time format similar to the ISO 8601 standard. */
  DateTime autoRenewStatusChangeDate = null;
  
/* The environment for which the receipt was generated. */
  String environment = null;
  //enum environmentEnum {  Sandbox,  PROD,  };
/* The reason a subscription expired. This field is only present for an expired auto-renewable subscription. See expiration_intent for more information. https://developer.apple.com/documentation/appstorereceipts/expiration_intent Possible Values  1: The customer voluntarily canceled their subscription. 2: Billing error; for example, the customer's payment information was no longer valid. 3: The customer did not agree to a recent price increase. 4: The product was not available for purchase at the time of renewal. 5: Unknown error.  */
  int expirationIntent = null;
  
/* The latest Base64-encoded transaction receipt. This field appears in the notification instead of latest_receipt for expired transactions. */
  String latestExpiredReceipt = null;
  

  List<Object> latestExpiredReceiptInfo = [];
  
/* The latest Base64-encoded transaction receipt. */
  String latestReceipt = null;
  
/* The JSON representation of the value in latest_receipt. Note that this field is an array in the receipt but a single object in server-to-server notifications. */
  List<AppleStoreLatestReceiptInfo> latestReceiptInfo = [];
  
/* The subscription event that triggered the notification. https://developer.apple.com/documentation/appstoreservernotifications/notification_type CANCEL: Indicates that either Apple customer support canceled the subscription or the user upgraded their subscription. The cancellation_date key contains the date and time of the change. DID_CHANGE_RENEWAL_PREF: Indicates the customer made a change in their subscription plan that takes effect at the next renewal. The currently active plan is not affected. DID_CHANGE_RENEWAL_STATUS: Indicates a change in the subscription renewal status. Check auto_renew_status_change_date_ms and auto_renew_status in the JSON response to know the date and time of the last status update and the current renewal status. DID_FAIL_TO_RENEW: Indicates a subscription that failed to renew due to a billing issue. Check is_in_billing_retry_period to know the current retry status of the subscription, and grace_period_expires_date to know the new service expiration date if the subscription is in a billing grace period. DID_RECOVER: Indicates successful automatic renewal of an expired subscription that failed to renew in the past. Check expires_date to determine the next renewal date and time. INITIAL_BUY: Occurs at the initial purchase of the subscription. Store latest_receipt on your server as a token to verify the user’s subscription status at any time, by validating it with the App Store. INTERACTIVE_RENEWAL: Indicates the customer renewed a subscription interactively, either by using your app’s interface, or on the App Store in the account's Subscriptions settings. Make service available immediately. RENEWAL: Indicates successful automatic renewal of an expired subscription that failed to renew in the past. Check expires_date to determine the next renewal date and time.  */
  String notificationType = null;
  //enum notificationTypeEnum {  CANCEL,  DID_CHANGE_RENEWAL_PREF,  DID_CHANGE_RENEWAL_STATUS,  DID_FAIL_TO_RENEW,  DID_RECOVER,  INITIAL_BUY,  INTERACTIVE_RENEWAL,  RENEWAL,  };
/* The same value as the shared secret you submit in the password field of the requestBody when validating receipts. */
  String password = null;
  
/* An object that contains information about the most recent in-app purchase transactions for the app. */
  AppleStoreUnifiedReceipt unifiedReceipt = null;
  
  AppleStoreServerNotification();

  @override
  String toString() {
    return 'AppleStoreServerNotification[autoRenewAdamId=$autoRenewAdamId, autoRenewProductId=$autoRenewProductId, autoRenewStatus=$autoRenewStatus, autoRenewStatusChangeDate=$autoRenewStatusChangeDate, environment=$environment, expirationIntent=$expirationIntent, latestExpiredReceipt=$latestExpiredReceipt, latestExpiredReceiptInfo=$latestExpiredReceiptInfo, latestReceipt=$latestReceipt, latestReceiptInfo=$latestReceiptInfo, notificationType=$notificationType, password=$password, unifiedReceipt=$unifiedReceipt, ]';
  }

  AppleStoreServerNotification.fromJson(Map<String, dynamic> json) {
    if (json == null) return;
    autoRenewAdamId =
        json['auto_renew_adam_id']
    ;
    autoRenewProductId =
        json['auto_renew_product_id']
    ;
    autoRenewStatus =
        json['auto_renew_status']
    ;
    autoRenewStatusChangeDate = json['auto_renew_status_change_date'] == null ? null : DateTime.parse(json['auto_renew_status_change_date']);
    environment =
        json['environment']
    ;
    expirationIntent =
        json['expiration_intent']
    ;
    latestExpiredReceipt =
        json['latest_expired_receipt']
    ;
    latestExpiredReceiptInfo =
      Object.listFromJson(json['latest_expired_receipt_info'])
;
    latestReceipt =
        json['latest_receipt']
    ;
    latestReceiptInfo =
      AppleStoreLatestReceiptInfo.listFromJson(json['latest_receipt_info'])
;
    notificationType =
        json['notification_type']
    ;
    password =
        json['password']
    ;
    unifiedReceipt =
      
      
      new AppleStoreUnifiedReceipt.fromJson(json['unified_receipt'])
;
  }

  Map<String, dynamic> toJson() {
    return {
      'auto_renew_adam_id': autoRenewAdamId,
      'auto_renew_product_id': autoRenewProductId,
      'auto_renew_status': autoRenewStatus,
      'auto_renew_status_change_date': autoRenewStatusChangeDate == null ? '' : autoRenewStatusChangeDate.toUtc().toIso8601String(),
      'environment': environment,
      'expiration_intent': expirationIntent,
      'latest_expired_receipt': latestExpiredReceipt,
      'latest_expired_receipt_info': latestExpiredReceiptInfo,
      'latest_receipt': latestReceipt,
      'latest_receipt_info': latestReceiptInfo,
      'notification_type': notificationType,
      'password': password,
      'unified_receipt': unifiedReceipt
     };
  }

  static List<AppleStoreServerNotification> listFromJson(List<dynamic> json) {
    return json == null ? new List<AppleStoreServerNotification>() : json.map((value) => new AppleStoreServerNotification.fromJson(value)).toList();
  }

  static Map<String, AppleStoreServerNotification> mapFromJson(Map<String, Map<String, dynamic>> json) {
    var map = new Map<String, AppleStoreServerNotification>();
    if (json != null && json.length > 0) {
      json.forEach((String key, Map<String, dynamic> value) => map[key] = new AppleStoreServerNotification.fromJson(value));
    }
    return map;
  }
}

